# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: Copyright 2023 Atmatk Techno Inc.

import sys
import dbus
from traceback import print_exc

DBUS_INTERFACE_PROPERTIES = 'org.freedesktop.DBus.Properties'
MM_DBUS_NAME = 'org.freedesktop.ModemManager1'
MM_DBUS_OBJ_PATH = '/org/freedesktop/ModemManager1'
MM_DBUS_OBJ_INTERFACE = 'org.freedesktop.DBus.ObjectManager'
MM_DBUS_INTERFACE_MODEM = 'org.freedesktop.ModemManager1.Modem'


class ModemNotFoundError(Exception):
    """Exception of Modem Not Found"""
    pass


class MMDBus:
    _dbus = None
    _obj = None

    def __init__(self):
        self._dbus = dbus.SystemBus()

        try:
            self._obj = self._dbus.get_object(MM_DBUS_NAME, MM_DBUS_OBJ_PATH)
        except dbus.DBusException:
            print_exc()
            raise

    def dbus(self):
        return self._dbus

    def obj(self):
        return self._obj


class MMModem:
    _mm_dbus = None
    _iface = None
    _modem_iface = None
    _obj = None
    _props_iface = None

    def __init__(self):
        self._mm_dbus = MMDBus()
        self._iface = dbus.Interface(
                                    self._mm_dbus.obj(),
                                    dbus_interface=MM_DBUS_OBJ_INTERFACE)
        modems = self._iface.GetManagedObjects()

        if not modems:
            print("no modems found")
            raise ModemNotFoundError('no modem found')

        for m in modems:
            self._obj = self._mm_dbus.dbus().get_object(MM_DBUS_NAME, m)
            if self._obj:
                print("first modem found")
                self._modem_iface = dbus.Interface(
                                    self._obj,
                                    dbus_interface=MM_DBUS_INTERFACE_MODEM)
                self._props_iface = dbus.Interface(
                                    self._obj,
                                    dbus_interface=DBUS_INTERFACE_PROPERTIES)
                break

        if not self._obj:
            print("no modem found")
            raise ModemNotFoundError('no modem found')

    def signal_quality(self):
        msg = self._obj.Get(MM_DBUS_INTERFACE_MODEM,
                            'SignalQuality',
                            dbus_interface=dbus.PROPERTIES_IFACE)
        if msg:
            return dbus.String(msg[0])
        else:
            return 0

    def imei(self):
        return self._obj.Get(MM_DBUS_INTERFACE_MODEM,
                             'EquipmentIdentifier',
                             dbus_interface=dbus.PROPERTIES_IFACE)

    def own_numbers(self):
        msg = self._obj.Get(MM_DBUS_INTERFACE_MODEM,
                            'OwnNumbers',
                            dbus_interface=dbus.PROPERTIES_IFACE)
        if msg:
            return msg[0]
        else:
            return None
